import 'package:flutter/material.dart';

// Tipos de animaciones
enum AnimationType { normal, fadeIn }

// Main class, [Context] es el BuildContext de la aplicación en ese momento
// [child] es el widget a navegar, [animation] es el tipo de animación
class RouteTransitions {
  final BuildContext context;
  final Widget child;
  final AnimationType animation;
  final Duration duration;
  final bool replacement;

  RouteTransitions(
      {@required this.context,
      @required this.child,
      this.animation = AnimationType.normal,
      this.duration = const Duration(milliseconds: 300),
      this.replacement = false}) {
    switch (this.animation) {
      case AnimationType.normal:
        final route = this._normalTransition();
        this._pushOrReplacePage(route);
        break;
      case AnimationType.fadeIn:
        final route = this._fadeInTransition();
        this._pushOrReplacePage(route);
        break;
    }
  }

  // Push or replace de la página
  void _pushOrReplacePage(Route route) {
    (this.replacement)
        ? this._pushReplacementPage(route)
        : this._pushPage(route);
  }

  void _pushPage(Route route) {
    Navigator.push(this.context, route);
  }

  void _pushReplacementPage(Route route) {
    Navigator.pushReplacement(this.context, route);
  }

  // Manejo de la transición normal
  PageRoute _normalTransition() {
    final route = MaterialPageRoute(builder: (_) => this.child);
    return route;
  }

  // Manejo de la transición con FadeIn
  PageRoute _fadeInTransition() {
    final route = PageRouteBuilder(
        pageBuilder: (_, __, ___) => this.child,
        transitionDuration: this.duration,
        transitionsBuilder: (_, animation, __, child) {
          return FadeTransition(
            child: child,
            opacity: Tween<double>(begin: 0.0, end: 1.0).animate(
                CurvedAnimation(parent: animation, curve: Curves.easeOut)),
          );
        });

    return route;
  }
}
